package gregapi.block.multitileentity;

import gregapi.block.multitileentity.IMultiTileEntity.IMTE_AddToolTips;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_CanPlace;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnPlaced;
import gregapi.item.IItemUpdatable;
import gregapi.util.UT;

import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 */
public class MultiTileEntityItemInternal extends ItemBlock implements IItemUpdatable {
	public final MultiTileEntityBlockInternal mBlock;
	
	public MultiTileEntityItemInternal(Block aBlock) {
		super(aBlock);
        setMaxDamage(0);
        setHasSubtypes(true);
		mBlock = (MultiTileEntityBlockInternal)aBlock;
	}
	
	@Override
	public void addInformation(ItemStack aStack, EntityPlayer aPlayer, List aList, boolean aF3_H) {
		TileEntity tTileEntity = mBlock.mMultiTileEntityRegistry.getNewTileEntity(aStack);
		if (tTileEntity instanceof IMTE_AddToolTips) ((IMTE_AddToolTips)tTileEntity).addToolTips(aList, aStack, aF3_H);
    }
	
	@Override
	@SideOnly(Side.CLIENT)
	public void getSubItems(Item aItem, CreativeTabs aTab, List aList) {
		for (MultiTileEntityClassContainer tClass : mBlock.mMultiTileEntityRegistry.mRegistrations) aList.add(new ItemStack(aItem, 1, tClass.mID));
	}
	
    @Override
	public boolean onItemUse(ItemStack aStack, EntityPlayer aPlayer, World aWorld, int aX, int aY, int aZ, int aSide, float aHitX, float aHitY, float aHitZ) {
        Block tClickedBlock = aWorld.getBlock(aX, aY, aZ);
        if (tClickedBlock == Blocks.snow_layer && (aWorld.getBlockMetadata(aX, aY, aZ) & 7) < 1) {
            aSide = 1;
        } else if (tClickedBlock != Blocks.vine && tClickedBlock != Blocks.tallgrass && tClickedBlock != Blocks.deadbush && !tClickedBlock.isReplaceable(aWorld, aX, aY, aZ)) {
            if (aSide == 0) aY--; else if (aSide == 1) aY++; else if (aSide == 2) aZ--; else if (aSide == 3) aZ++; else if (aSide == 4) aX--; else if (aSide == 5) aX++;
        }
        
        if (aStack.stackSize == 0 || !aPlayer.canPlayerEdit(aX, aY, aZ, aSide, aStack) || aY == 255) return false;
        
        MultiTileEntityContainer tContainer = mBlock.mMultiTileEntityRegistry.getNewTileEntityContainer(aStack);
        
        if (tContainer != null && aWorld.canPlaceEntityOnSide(tContainer.mBlock, aX, aY, aZ, false, aSide, aPlayer, aStack) && (!(tContainer.mTileEntity instanceof IMTE_CanPlace) || ((IMTE_CanPlace)tContainer.mTileEntity).canPlace(aStack, aPlayer, aWorld, aX, aY, aZ, (byte)aSide, aHitX, aHitY, aHitZ)) && aWorld.setBlock(aX, aY, aZ, tContainer.mBlock, tContainer.mBlockMetaData, 3)) {
        	UT.Worlds.setTileEntity(aWorld, aX, aY, aZ, tContainer.mTileEntity, true);
        	if (!(tContainer.mTileEntity instanceof IMTE_OnPlaced) || ((IMTE_OnPlaced)tContainer.mTileEntity).onPlaced(aStack, aPlayer, aWorld, aX, aY, aZ, (byte)aSide, aHitX, aHitY, aHitZ)) {
        		aWorld.playSoundEffect((aX + 0.5F), (aY + 0.5F), (aZ + 0.5F), tContainer.mBlock.stepSound.func_150496_b(), (tContainer.mBlock.stepSound.getVolume() + 1.0F) / 2.0F, tContainer.mBlock.stepSound.getPitch() * 0.8F);
        	}
    		aStack.stackSize--;
    		return true;
        }
		return false;
    }
    
	@Override
	public void updateItemStack(ItemStack aStack) {
		//
	}
	
	@Override
	public int getItemStackLimit(ItemStack aStack) {
		MultiTileEntityClassContainer tContainer = mBlock.mMultiTileEntityRegistry.getClassContainer(aStack);
		return tContainer == null ? 1 : tContainer.mStackSize;
	}
	
	@Override public final ItemBlock setUnlocalizedName(String aName) {return this;}
	@Override public final String getUnlocalizedName() {return mBlock.mMultiTileEntityRegistry.mNameInternal;}
	@Override public final String getUnlocalizedName(ItemStack aStack) {return mBlock.mMultiTileEntityRegistry.mNameInternal+"."+getDamage(aStack);}
	@Override public final boolean hasContainerItem(ItemStack aStack) {return getContainerItem(aStack) != null;}
	@Override public ItemStack getContainerItem(ItemStack aStack) {return null;}
    @Override public int getSpriteNumber() {return 0;}
    @Override public void registerIcons(IIconRegister p_94581_1_) {/**/}
	@Override public boolean isBookEnchantable(ItemStack aStack, ItemStack aBook) {return false;}
	@Override public boolean getIsRepairable(ItemStack aStack, ItemStack aMaterial) {return false;}
	@Override public int getItemEnchantability() {return 0;}
	@Override public void onCreated(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {updateItemStack(aStack);}
}