package gregapi.tileentity;

import gregapi.GT_API;
import gregapi.util.UT;
import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.IInventory;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.fluids.IFluidHandler;

/**
 * @author Gregorius Techneticies
 * 
 * The Functions all TileEntities should have.
 */
public abstract class TileEntityBase1 extends TileEntity implements ITileEntity {
	/** If this TileEntity checks for the Chunk to be loaded before returning World based values. If this is set to true, this TileEntity will not cause worfin' Chunks, uhh I mean orphan Chunks. */
	public boolean mIgnoreUnloadedChunks = true;
	
	/** This Variable checks if this TileEntity is dead, because Minecraft is too stupid to have proper TileEntity unloading. */
	public boolean mIsDead = false;
	
	/** This Variable checks if this TileEntity should refresh when the Block is being set. That way you can turn this check off anytime you need it. */
	public boolean mShouldRefresh = true;
	
	/** If this TileEntity is ticking at all */
	public final boolean mIsTicking;
	
	public TileEntityBase1(boolean aIsTicking) {
		mIsTicking = aIsTicking;
	}
	
    /**
     * YOU MUST HAVE THIS INSIDE YOUR BLOCK CODE!!!
     * 
     * public void onNeighborChange(IBlockAccess aWorld, int aX, int aY, int aZ, int aTileX, int aTileY, int aTileZ) {
     *     TileEntity tTileEntity = aWorld.getTileEntity(aX, aY, aZ);
     *     if (tTileEntity instanceof ITileEntity) ((ITileEntity)tTileEntity).onAdjacentBlockChange(aTileX, aTileY, aTileZ);
     * }
     * 
	 * public void onNeighborBlockChange(World aWorld, int aX, int aY, int aZ, Block aBlock) {
	 * 	   TileEntity tTileEntity = aWorld.getTileEntity(aX, aY, aZ);
	 *     if (tTileEntity instanceof ITileEntity) ((ITileEntity)tTileEntity).onAdjacentBlockChange(aX, aY, aZ);
     * }
     */
	@Override
	public void onAdjacentBlockChange(int aTileX, int aTileY, int aTileZ) {
		//
    }
	
	@Override public World getWorld() {return worldObj;}
	@Override public int getX() {return xCoord;}
	@Override public int getY() {return yCoord;}
	@Override public int getZ() {return zCoord;}
	@Override public int getOffsetX(byte aSide, int aMultiplier) {return xCoord + ForgeDirection.getOrientation(aSide).offsetX * aMultiplier;}
    @Override public int getOffsetY(byte aSide, int aMultiplier) {return yCoord + ForgeDirection.getOrientation(aSide).offsetY * aMultiplier;}
    @Override public int getOffsetZ(byte aSide, int aMultiplier) {return zCoord + ForgeDirection.getOrientation(aSide).offsetZ * aMultiplier;}
    @Override public boolean isServerSide() {return !worldObj.isRemote;}
	@Override public boolean isClientSide() {return  worldObj.isRemote;}
	@Override public boolean openGUI(EntityPlayer aPlayer) {return openGUI(aPlayer, 0);}
	@Override public boolean openGUI(EntityPlayer aPlayer, int aID) {if (aPlayer == null) return false; aPlayer.openGui(GT_API.instance, aID, worldObj, xCoord, yCoord, zCoord); return true;}
    @Override public int getRandomNumber(int aRange) {return worldObj.rand.nextInt(aRange);}
	@Override public BiomeGenBase getBiome(int aX, int aZ) {return worldObj.getBiomeGenForCoords(aX, aZ);}
	@Override public BiomeGenBase getBiome() {return getBiome(xCoord, zCoord);}
    @Override public Block getBlockOffset(int aX, int aY, int aZ) {return getBlock(xCoord+aX, yCoord+aY, zCoord+aZ);}
    @Override public Block getBlockAtSide(byte aSide) {return getBlockAtSideAndDistance(aSide, 1);}
    @Override public Block getBlockAtSideAndDistance(byte aSide, int aDistance) {return getBlock(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
    @Override public byte getMetaDataOffset(int aX, int aY, int aZ) {return getMetaData(xCoord+aX, yCoord+aY, zCoord+aZ);}
    @Override public byte getMetaDataAtSide(byte aSide) {return getMetaDataAtSideAndDistance(aSide, 1);}
    @Override public byte getMetaDataAtSideAndDistance(byte aSide, int aDistance) {return getMetaData(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
    @Override public byte getLightLevelOffset(int aX, int aY, int aZ) {return getLightLevel(xCoord+aX, yCoord+aY, zCoord+aZ);}
	@Override public byte getLightLevelAtSide(byte aSide) {return getLightLevelAtSideAndDistance(aSide, 1);}
	@Override public byte getLightLevelAtSideAndDistance(byte aSide, int aDistance) {return getLightLevel(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
	@Override public boolean getOpacityOffset(int aX, int aY, int aZ) {return getOpacity(xCoord+aX, yCoord+aY, zCoord+aZ);}
	@Override public boolean getOpacityAtSide(byte aSide) {return getOpacityAtSideAndDistance(aSide, 1);}
	@Override public boolean getOpacityAtSideAndDistance(byte aSide, int aDistance) {return getOpacity(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
	@Override public boolean getSkyOffset(int aX, int aY, int aZ) {return getSky(xCoord+aX, yCoord+aY, zCoord+aZ);}
	@Override public boolean getSkyAtSide(byte aSide) {return getSkyAtSideAndDistance(aSide, 1);}
	@Override public boolean getSkyAtSideAndDistance(byte aSide, int aDistance) {return getSky(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
	@Override public boolean getAirOffset(int aX, int aY, int aZ) {return getAir(xCoord+aX, yCoord+aY, zCoord+aZ);}
	@Override public boolean getAirAtSide(byte aSide) {return getAirAtSideAndDistance(aSide, 1);}
	@Override public boolean getAirAtSideAndDistance(byte aSide, int aDistance) {return getAir(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
	@Override public TileEntity getTileEntityOffset(int aX, int aY, int aZ) {return getTileEntity(xCoord+aX, yCoord+aY, zCoord+aZ);}
    @Override public TileEntity getTileEntityAtSideAndDistance(byte aSide, int aDistance) {return getTileEntity(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
    @Override public DelegatorTileEntity<TileEntity			> getAdjacentTileEntity	(byte aSide) {TileEntity tTileEntity = getTileEntityAtSideAndDistance(aSide, 1); return tTileEntity instanceof ITileEntityDelegating ? ((ITileEntityDelegating)tTileEntity).getDelegateTileEntity(UT.Code.opposite(aSide)) : new DelegatorTileEntity<TileEntity>(tTileEntity, UT.Code.opposite(aSide));}
	@Override public DelegatorTileEntity<IInventory			> getAdjacentInventory	(byte aSide) {DelegatorTileEntity tDelegator = getAdjacentTileEntity(aSide); return new DelegatorTileEntity<IInventory			>(tDelegator.mTileEntity instanceof IInventory			?(IInventory		)tDelegator.mTileEntity : null, tDelegator.mSideOfTileEntity);}
    @Override public DelegatorTileEntity<IFluidHandler		> getAdjacentTank		(byte aSide) {DelegatorTileEntity tDelegator = getAdjacentTileEntity(aSide); return new DelegatorTileEntity<IFluidHandler		>(tDelegator.mTileEntity instanceof IFluidHandler		?(IFluidHandler		)tDelegator.mTileEntity : null, tDelegator.mSideOfTileEntity);}
    @Override public DelegatorTileEntity<ITileEntityEnergy	> getAdjacentEnergyTile	(byte aSide) {DelegatorTileEntity tDelegator = getAdjacentTileEntity(aSide); return new DelegatorTileEntity<ITileEntityEnergy	>(tDelegator.mTileEntity instanceof ITileEntityEnergy	?(ITileEntityEnergy	)tDelegator.mTileEntity : null, tDelegator.mSideOfTileEntity);}
    
    @Override
    public Block getBlock(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return Blocks.air;
    	return worldObj.getBlock(aX, aY, aZ);
    }
    
    @Override
    public byte getMetaData(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return 0;
    	return (byte)worldObj.getBlockMetadata(aX, aY, aZ);
    }
    
    @Override
    public byte getLightLevel(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return 0;
    	return (byte)(worldObj.getLightBrightness(aX, aY, aZ)*15);
    }
    
    @Override
    public boolean getSky(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return true;
    	return worldObj.canBlockSeeTheSky(aX, aY, aZ);
    }
    
    @Override
    public boolean getOpacity(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return false;
    	return worldObj.getBlock(aX, aY, aZ).isOpaqueCube();
    }
    
    @Override
    public boolean getAir(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return true;
    	return worldObj.getBlock(aX, aY, aZ).isAir(worldObj, aX, aY, aZ);
    }
    
    @Override
    public TileEntity getTileEntity(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return null;
    	return UT.Worlds.getTileEntity(worldObj, aX, aY, aZ, true);
    }
    
	@Override
	public void sendBlockEvent(byte aID, byte aValue) {
		worldObj.addBlockEvent(xCoord, yCoord, zCoord, getBlockOffset(0, 0, 0), aID, aValue);
	}
	
    @Override
    public boolean isDead() {
    	return mIsDead;
    }
    
    @Override
    public void invalidate() {
    	super.invalidate();
    	mIsDead = true;
    }
    
    @Override
    public void validate() {
    	super.validate();
    	mIsDead = false;
    }
    
    @Override
    public void onChunkUnload() {
    	super.onChunkUnload();
    	mIsDead = true;
    }
    
    @Override
    public void updateEntity() {
    	// Well, if the TileEntity gets ticked it is alive.
    	mIsDead = false;
    }
    
	@Override
	public long getTimer() {
		return 0;
	}
	
	@Override
    public boolean canUpdate() {
        return mIsTicking;
    }
	
    @Override
    public boolean shouldRefresh(Block aOldBlock, Block aNewBlock, int aOldMeta, int aNewMeta, World aWorld, int aX, int aY, int aZ) {
        return mShouldRefresh;
    }
	
	protected boolean crossedChunkBorder(int aX, int aZ) {
		return aX >> 4 != xCoord >> 4 || aZ >> 4 != zCoord >> 4;
	}
}